##############################################################################
# DynamiXer
# version 1.1
#
# created by Mayec Rancel
# http://www.mayec.eu
#
# PURPOSE:
# =======
# DynamiXer is a Maya UI tool which provides centralized, easy control of
# visibility & simulation for all the dynamics nodes (particles & fluids)
# in a scene.
#
# This can be specially useful to work with complex effects rigs, where
# several particle systems and/or fluids interact between them. Isolating
# some of them in either simulation, preview, or both, can be a good way to
# optimize your workflow, for speed and/or viewport clarity.
#
# TO DO:
# =====
#   - automatic refresh when:
#       when new node created
#       when node deleted
#       when node renamed
#       when new scene opened
#	- option to hide/unhide all fluids container boxes for easy and quick preview/playblasting.
##############################################################################

import maya.cmds as cmds
from functools import partial

autoRefresh = False
inDev = False

class dynamixerUI:
    version = 1.1
    maxScrollHeight = 650     # maximum pixel height of scrolling area that lists nodes
    nodeTypes = ['particle', 'fluidShape']
    nodeLabels = ['particle', 'fluid']
    # following dictionary defines attribute that controls simulation depending on nodeType(key)
    simAttribute = {'particle' : 'isDynamic',
               'fluidShape': 'disableInteractiveEval',
    }
    # following dictionary will give a True value if that nodeType's attribute works as reverse (true disables sim)
    reverse = {'particle' : False,
                'fluidShape' : True,
    }
    colorOn = [0,1,0]
    colorOff = [1,0,0]
        
    nodesList = {}
    

    def __init__(self):
        self.name = 'dynamixer'
        self.title = 'DynamiXer v'+str(self.version)
 
        # Keep all the iportant UI elements in a dictionary called UIElements
    	self.UIElements = {}
        
        self.resetWindow()
        self.createWindow()
        
        # update button colors according to actual status
        self.updateAllButtons() 
        

   #-------------------- UI creation ------------------------ 
   
    def refreshUI(self,args):
        prevWindowPos = cmds.window( self.window, query=True, topLeftCorner=True )
        self.resetWindow()
        self.createWindow()
        prevWindowPos = cmds.window( self.window, edit=True, topLeftCorner=prevWindowPos )
        # update button colors according to actual status
        self.updateAllButtons() 
   
    def resetWindow(self):
    	# Reset window preferences and delete previous instances of the window
    	if ( cmds.windowPref( self.name, exists=True ) ):
    	    cmds.windowPref( self.name, remove=True )
    	if (cmds.window(self.name, q=1, exists=True)):
    	    cmds.deleteUI(self.name)
    	    cmds.windowPref( self.name, remove=True )
    	    print("deletedWindow")

    def createWindow(self):
    	# Create the window element & main layout
    	self.UIElements['window'] = self.window = cmds.window( self.name, title = self.title )
        self.form = cmds.formLayout()
        
        # Create the basic general controls
        self.UIElements['displayAll'] = cmds.button(label='Display All', width=100, command=partial( self.setNodesDisplay, 'all', 1 ))        
        self.UIElements['displayNone'] = cmds.button(label='Display None', width=100, command=partial( self.setNodesDisplay, 'all', 0 ))
        self.UIElements['simAll'] = cmds.button(label='Simulate All', width=100, command=partial( self.setNodesSim, 'all', 1 ))        
        self.UIElements['simNone'] = cmds.button(label='Simulate None', width=100, command=partial( self.setNodesSim, 'all', 0 ))
        self.UIElements['refresh'] = cmds.button(label='Refresh', width=100, command=self.refreshUI )
        self.UIElements['about'] = cmds.frameLayout (label="About", width=300, collapsable=True, collapse=True , borderStyle='etchedOut', preExpandCommand=partial(self.adaptWindowHeight, 180), preCollapseCommand=partial(self.adaptWindowHeight, -180))
        cmds.scrollField( editable=False,  height=180, wordWrap=True, text=(
        self.title+"\n" +
        "by Mayec Rancel\n" +
        "\n" +
        "This UI tool provides centralized, easy control of visibility & simulation for all the dynamics nodes (particles & fluids) in a scene.\n\n" +
        "See dynamixer.txt file or script's comments for more detailed instructions.")
		)
        
        # Dynamically create the scrollable area with node groups by type
        self.UIElements['scroll'] = cmds.scrollLayout(parent=self.form, width = 320)
        for nodeTypeId in range(0, len(self.nodeTypes)):
            self.createNodeGroup( int(nodeTypeId))
        
        # Attach & arrange controls    
        cmds.formLayout( self.form,
            edit=True,
            attachForm = [
                (self.UIElements['displayAll'], 'top', 7),
                (self.UIElements['displayAll'], 'left', 20),
                (self.UIElements['displayNone'], 'left', 20),
                (self.UIElements['simAll'], 'right', 20),
                (self.UIElements['simNone'], 'right', 20),
                (self.UIElements['simAll'], 'top', 7),
                (self.UIElements['scroll'], 'left', 5),
                (self.UIElements['refresh'], 'left', 115),
                (self.UIElements['about'], 'left', 15),
                ],
            attachControl=[
                (self.UIElements['displayNone'], 'top', 5, self.UIElements['displayAll']),
                (self.UIElements['simNone'], 'top', 5, self.UIElements['simAll']),
                (self.UIElements['scroll'], 'top', 7, self.UIElements['simNone']),
                (self.UIElements['refresh'], 'top', 5, self.UIElements['scroll']),
                (self.UIElements['about'], 'top', 7, self.UIElements['refresh']),
            ]
        )         
        
       	# Dynamically adjust height of scroll area and window
       	fixHeightPx = 130
       	groupHeightPx = 48
       	nodeHeightPx = 26
       	totalNodes = 0
       	for i in range(0, len(self.nodesList)):
       	    totalNodes += len (self.nodesList[i])
       	dynamicHeightPx = len(self.nodeTypes) * groupHeightPx + totalNodes * nodeHeightPx
       	if (dynamicHeightPx > self.maxScrollHeight): dynamicHeightPx = self.maxScrollHeight
        # Adjust height of scroll area
       	cmds.scrollLayout( self.UIElements['scroll'], edit=True, height = dynamicHeightPx )
       	# Adjust height of window
       	windowHeight = fixHeightPx + dynamicHeightPx  
        self.window = cmds.window( self.name, e=1, widthHeight=(330,windowHeight) )
        cmds.showWindow (self.window)
        
        if autoRefresh:
            # Create scriptJob for automatic refresh
            cmds.scriptJob( event=['SelectionChanged','dynamixerUI.refreshUI(0)'], parent = self.window)
        
    def createNodeGroup( self, nodeTypeId ):
        nodeType = self.nodeTypes[nodeTypeId]
        nodeLabel = self.nodeLabels[nodeTypeId]
        self.UIElements[nodeType+'sFrame'] = cmds.frameLayout( parent=self.UIElements['scroll'], label=(nodeLabel+'s').capitalize(), collapsable=True, width=296 )
        self.UIElements[nodeType+'sForm'] = cmds.formLayout()
        self.UIElements[nodeType+'sDispAll'] = cmds.button( label='Displ.All', width=58, height = 20, command=partial( self.setNodesDisplay, nodeType, 1 ) )
        self.UIElements[nodeType+'sDispNone'] = cmds.button( label='Displ.None', width=58, height = 20, command=partial( self.setNodesDisplay, nodeType, 0 ) )
        self.UIElements[nodeType+'sSimAll'] = cmds.button( label='Sim All', width=58, height = 20, command=partial( self.setNodesSim, nodeType, 1 ) )
        self.UIElements[nodeType+'sSimNone'] = cmds.button( label='Sim None', width=58, height = 20, command=partial( self.setNodesSim, nodeType, 0 ) )
        self.UIElements[nodeType+'sList'] = cmds.columnLayout( columnAttach=('both', 5), columnWidth=290 )
        self.nodesList[nodeTypeId] = cmds.ls( type=nodeType )
        for i in range(0, len(self.nodesList[nodeTypeId]) ):
            self.createNodeRow( self.nodesList[nodeTypeId][i], nodeType, i)
        # Attach & arrange controls    
        cmds.formLayout( self.UIElements[nodeType+'sForm'],
            edit=True,
            attachForm = [
                (self.UIElements[nodeType+'sDispAll'], 'left', 20),
                (self.UIElements[nodeType+'sSimNone'], 'right', 20),
                ],
            attachControl=[
                (self.UIElements[nodeType+'sDispNone'], 'left', 2, self.UIElements[nodeType+'sDispAll']),
                (self.UIElements[nodeType+'sSimAll'], 'right', 2, self.UIElements[nodeType+'sSimNone']),
                (self.UIElements[nodeType+'sList'], 'top', 2, self.UIElements[nodeType+'sDispNone']),
            ]
        ) 
            
            
    def createNodeRow( self, theNode, nodeType, i ):
        transformNode = str( cmds.listRelatives(theNode, parent=True)[0] )
        self.UIElements[nodeType+'Form' + str(i)] = cmds.formLayout( parent= self.UIElements[nodeType+'sList'] )
        self.UIElements[nodeType+'Label' + str(i)] = cmds.text( label = str(transformNode) )
        self.UIElements[nodeType+'DispBtn' + str(i)] = cmds.button( label='V', width=26, command=partial( self.tweakDisplay, theNode, nodeType, i) )
        self.UIElements[nodeType+'SimBtn' + str(i)] = cmds.button( label='S', width=26, command=partial( self.tweakSim, theNode, nodeType, i) )
        # layout row elements
        cmds.formLayout( self.UIElements[nodeType+'Form' + str(i)],
            edit=True,
            attachForm = [
                (self.UIElements[nodeType+'Label' + str(i)], 'top', 5),
                (self.UIElements[nodeType+'SimBtn' + str(i)], 'right', 0),
                ],
            attachControl=[
                (self.UIElements[nodeType+'DispBtn' + str(i)], 'right', 5, self.UIElements[nodeType+'SimBtn' + str(i)]),                
            ]
        )        


    def updateAllButtons(self, *args):
        # Updates color of all buttons
        # iterate NodeTypes
        for nodeTypeId in range(0, len(self.nodeTypes) ):
            nodeType = self.nodeTypes[nodeTypeId]
            # iterate nodes of that NodeType
            for i in range(0, len(self.nodesList[nodeTypeId]) ):
                theNode = self.nodesList[nodeTypeId][i]
                # Update VISIBILITY button
                attribute = 'visibility'
                if (cmds.getAttr(theNode+'.'+attribute)):
                    cmds.button( self.UIElements[nodeType+'DispBtn' + str(i)], e=1, backgroundColor=self.colorOn )
                else:
                    cmds.button( self.UIElements[nodeType+'DispBtn' + str(i)], e=1, backgroundColor=self.colorOff )
                # Update SIM button
                attribute = self.simAttribute[nodeType]
                reverse = self.reverse[nodeType]
                # invert sim attribute value and change button color accordingly
                if (cmds.getAttr(theNode+'.'+attribute)):
                    if reverse:
                        cmds.button( self.UIElements[nodeType+'SimBtn' + str(i)], e=1, backgroundColor=self.colorOff )
                    else:
                        cmds.button( self.UIElements[nodeType+'SimBtn' + str(i)], e=1, backgroundColor=self.colorOn )
                else:
                    if reverse:
                        cmds.button( self.UIElements[nodeType+'SimBtn' + str(i)], e=1, backgroundColor=self.colorOn )
                    else:
                        cmds.button( self.UIElements[nodeType+'SimBtn' + str(i)], e=1, backgroundColor=self.colorOff )
                        
    
    def adaptWindowHeight(self, offset, *args):
        sameWidth = cmds.window( self.name, q=True, width=True)
        newHeight = cmds.window( self.name, q=True, height=True) + offset
        self.window = cmds.window( self.name, e=1, widthHeight=(sameWidth, newHeight) )
                               
    #----------------------- UI Functionality -----------------------
    
    
    def tweakDisplay( self, theNode, nodeType, i, arg=None ):
        attribute = 'visibility'
        # invert 'visibility' value and change button color accordingly
        if (cmds.getAttr(theNode+'.'+attribute)):
            cmds.setAttr ( (theNode+'.'+attribute), 0)
            cmds.button( self.UIElements[nodeType+'DispBtn' + str(i)], e=1, backgroundColor=self.colorOff )
        else:
            cmds.setAttr ( (theNode+'.'+attribute), 1)
            cmds.button( self.UIElements[nodeType+'DispBtn' + str(i)], e=1, backgroundColor=self.colorOn )

    
    def tweakSim(  self, theNode, nodeType, i, arg=None ):
        attribute = self.simAttribute[nodeType]
        reverse = self.reverse[nodeType]
        # invert sim attribute value and change button color accordingly
        if (cmds.getAttr(theNode+'.'+attribute)):
            cmds.setAttr ( (theNode+'.'+attribute), 0)
        else:
            cmds.setAttr ( (theNode+'.'+attribute), 1)
        self.updateAllButtons()

    
    def setNodesDisplay( self, nodeType, newState, arg=None ):
        if (nodeType == 'all'):
            actOnTypes = self.nodeTypes
        else:
            actOnTypes = [nodeType]
        for actNodeType in actOnTypes:
            theNodes = cmds.ls (type = actNodeType)
            for node in theNodes:
                cmds.setAttr ( (node+'.visibility'), newState);
        self.updateAllButtons()
            
    def setNodesSim( self, nodeType, newState, arg=None ):
        if (nodeType == 'all'):
            actOnTypes = self.nodeTypes
        else:
            actOnTypes = [nodeType]
        for actNodeType in actOnTypes:
            theNodes = cmds.ls (type = actNodeType)
            attribute = self.simAttribute[actNodeType]
            reverse = self.reverse[actNodeType]
            if (reverse):
                newState = not newState
            for node in theNodes:
                cmds.setAttr ( (node+'.'+attribute), newState);
        self.updateAllButtons()
#------- END Class myUI --------

dynamixerUI = dynamixerUI()